package com.rjconsultores.ventaboletos.web.gui.controladores.relatorios;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.InputStream;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;

import javax.sql.DataSource;

import org.apache.log4j.Logger;
import org.apache.poi.hssf.usermodel.HSSFCell;
import org.apache.poi.hssf.usermodel.HSSFCellStyle;
import org.apache.poi.hssf.usermodel.HSSFFont;
import org.apache.poi.hssf.usermodel.HSSFRow;
import org.apache.poi.hssf.usermodel.HSSFSheet;
import org.apache.poi.hssf.usermodel.HSSFWorkbook;
import org.apache.poi.ss.usermodel.CellStyle;
import org.apache.poi.ss.util.CellRangeAddress;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.zkoss.zhtml.Filedownload;
import org.zkoss.zk.ui.Component;
import org.zkoss.zk.ui.event.Event;
import org.zkoss.zul.Datebox;
import org.zkoss.zul.ListModelList;
import org.zkoss.zul.Timebox;

import com.rjconsultores.ventaboletos.entidad.Empresa;
import com.rjconsultores.ventaboletos.entidad.OrgaoConcedente;
import com.rjconsultores.ventaboletos.entidad.Parada;
import com.rjconsultores.ventaboletos.entidad.Ruta;
import com.rjconsultores.ventaboletos.relatorios.utilitarios.RelatorioEmbarqueLocalidadeBean;
import com.rjconsultores.ventaboletos.service.EmpresaService;
import com.rjconsultores.ventaboletos.service.OrgaoConcedenteService;
import com.rjconsultores.ventaboletos.service.ParadaService;
import com.rjconsultores.ventaboletos.service.RutaService;
import com.rjconsultores.ventaboletos.utilerias.UsuarioLogado;
import com.rjconsultores.ventaboletos.web.utilerias.MyComboboxEstandar;
import com.rjconsultores.ventaboletos.web.utilerias.MyGenericForwardComposer;
import com.rjconsultores.ventaboletos.web.utilerias.NamedParameterStatement;

@Controller("relatorioEmbarqueLocalidadeController")
public class RelatorioEmbarqueLocalidadeController  extends MyGenericForwardComposer {
	private static final long serialVersionUID = 1L;
	
	private static Logger log = Logger.getLogger(RelatorioEmbarqueLocalidadeController.class);

	@Autowired
	private DataSource dataSourceRead;
	@Autowired 
	private EmpresaService empresaService;
	@Autowired 
	private RutaService rutaService;
	@Autowired 
	private ParadaService paradaService;
	@Autowired 
	private OrgaoConcedenteService orgaoConcedenteService;
	
	private Datebox datInicial;
	private Datebox datFinal;
	private Timebox horInicial;
	private Timebox horFinal;
	private MyComboboxEstandar cmbEmpresa;
	private MyComboboxEstandar cmbOrgaoConcedente;
	private MyComboboxEstandar cmbRuta;
	private MyComboboxEstandar cmbOrigem;
	
	private List<Empresa> lsEmpresas;
	private List<OrgaoConcedente> lsOrgaosConcedentes;
	private List<Ruta> lsRutas;
	private List<Parada> lsOrigens;
	
	protected static final String FIELD_FEC_FINAL = "fecFinal";
	protected static final String FILED_FEC_INICIO = "fecInicio";
	protected static final String FIELD_HOR_INICIO = "horInicio";
	protected static final String FIELD_HOR_FINAL = "horFinal";
	protected static final String FIELD_EMPRESA_ID = "empresa";
	protected static final String FIELD_RUTA_ID = "ruta";
	protected static final String FIELD_ORGAO_CONCEDENTE_ID = "orgao";
	protected static final String FIELD_ORIGEM_ID = "origem";
	
	private static final int NUM_LINHA_TITULO = 0;
	private static final int NUM_LINHA_PERIODO = 1;
	private static final int NUM_LINHA_DATA_EMPRESA = 3;
	private static final int NUM_LINHA_INICIO = 5;
	 
	private Empresa empresa;
	private Ruta ruta;
	private OrgaoConcedente orgao; 
	private Parada origem;
		
	
	private List<RelatorioEmbarqueLocalidadeBean> lsDadosRelatorio;
	
	private void executarRelatorio() throws Exception {
		
		empresa =  cmbEmpresa.getSelectedItem() != null ? (Empresa)cmbEmpresa.getSelectedItem().getValue() : null;
		ruta = cmbRuta.getSelectedItem() != null ? (Ruta)cmbRuta.getSelectedItem().getValue() : null;
		orgao = cmbOrgaoConcedente.getSelectedItem() != null ? (OrgaoConcedente)cmbOrgaoConcedente.getSelectedItem().getValue() : null; 
		origem = cmbOrigem.getSelectedItem() != null ? (Parada)cmbOrigem.getSelectedItem().getValue() : null;
		
		SimpleDateFormat sdf = new SimpleDateFormat("dd/MM/yyyy");
		SimpleDateFormat hf = new SimpleDateFormat("HH:mm:ss");
		String fecInicio = sdf.format(this.datInicial.getValue()) + " 00:00:00";
		String fecFinal = sdf.format(this.datFinal.getValue()) + " 23:59:59";
		String horInicio = horInicial.getValue() != null ? hf.format(horInicial.getValue()) : "";
		String horFim = horFinal.getValue() != null ? hf.format(horFinal.getValue()) : "";
		
		
		
		String sql = getSql(empresa.getEmpresaId().toString(), 
				ruta != null ? ruta.getRutaId().toString() : null, 
				orgao != null ? orgao.getOrgaoConcedenteId().toString() : null, 
				origem != null ? origem.getParadaId().toString() : null, horInicio, horFim);
		
		try{
			NamedParameterStatement stmt = new NamedParameterStatement(dataSourceRead.getConnection(), sql);
			ResultSet rset = null;
	
			stmt.setString(FILED_FEC_INICIO, fecInicio);
			stmt.setString(FIELD_FEC_FINAL, fecFinal);
			if (empresa != null) {
				stmt.setInt(FIELD_EMPRESA_ID, empresa.getEmpresaId());
			}
	
			if (ruta != null && !ruta.equals("-1")) {
				stmt.setInt(FIELD_RUTA_ID, ruta.getRutaId());
			}
			
			if (orgao != null) {
				stmt.setInt(FIELD_ORGAO_CONCEDENTE_ID, orgao.getOrgaoConcedenteId());
			}
			
			if (origem != null) {
				stmt.setInt(FIELD_ORIGEM_ID, origem.getParadaId());
			}
			
			if (!horInicio.isEmpty() && !horFim.isEmpty()){
				stmt.setString(FIELD_HOR_INICIO, horInicio);
				stmt.setString(FIELD_HOR_FINAL, horFim);
			}
			rset = stmt.executeQuery();
			
			lsDadosRelatorio = new ArrayList<RelatorioEmbarqueLocalidadeBean>();
			Map<String, Map<String, List<RelatorioEmbarqueLocalidadeBean>>> datas = ResultSetToList(rset);
			
			exportaExcel(datas);
		} catch (Exception e){
			log.error("", e);
		}		
	}
	
	private void exportaExcel(Map<String, Map<String, List<RelatorioEmbarqueLocalidadeBean>>> datas){
		try{
			HSSFWorkbook wb = new HSSFWorkbook();
	        
	        HSSFCellStyle estiloVerticalMerge = wb.createCellStyle();
	        estiloVerticalMerge.setVerticalAlignment(CellStyle.VERTICAL_CENTER);
	        
	        HSSFSheet sheet1 = null;
	
	        int linha = NUM_LINHA_INICIO;
	        int coluna = 1;
	        HSSFRow row = null;
	        
	        for (Entry<String, Map<String, List<RelatorioEmbarqueLocalidadeBean>>> data : datas.entrySet()){
	        	// entry.getKey() => Data
	        	sheet1 = wb.createSheet(data.getKey().replaceAll("/", "-"));
	        	geraCabecalho(sheet1, data.getKey(), wb);
	        	int totalGeral = 0;
	        	for (Entry<String, List<RelatorioEmbarqueLocalidadeBean>> horario : data.getValue().entrySet()){
		        	row = sheet1.createRow(linha);
			        HSSFCell cellHorario = row.createCell(0);
			        cellHorario.setCellValue(horario.getKey());	
			        cellHorario.setCellStyle(estiloVerticalMerge);
			        
			        HSSFCell cellTotal = row.createCell(7);
			        
			        List<RelatorioEmbarqueLocalidadeBean> rows = horario.getValue();
			        
			        int rowcount = (int) Math.ceil((double)rows.size() / 3) -1;
			        
			        
			        sheet1.addMergedRegion(new CellRangeAddress(
							linha, //first row (0-based)
							linha+rowcount, //last row  (0-based)
				            0, //first column (0-based)
				            0  //last column  (0-based)
				    ));
			        
			        sheet1.addMergedRegion(new CellRangeAddress(
							linha, //first row (0-based)
							linha+rowcount, //last row  (0-based)
				            7, //first column (0-based)
				            7  //last column  (0-based)
				    ));
			        
			        int beancount = 1;
			        int embarquesTotais = 0;
			        for (RelatorioEmbarqueLocalidadeBean bean : rows){
			        	if (beancount == 4){
			        		linha++;
			        		row = sheet1.createRow(linha);
			        		coluna = 1;
			        	}
			        	row.createCell(coluna++).setCellValue(bean.getLinha());
			        	row.createCell(coluna++).setCellValue(bean.getEmbarques());
			        	embarquesTotais += bean.getEmbarques();
			        	beancount++;
			        }
			        
			        cellTotal.setCellValue(embarquesTotais);	
			        cellTotal.setCellStyle(estiloVerticalMerge);
			        
			        linha++;
			        coluna = 1;
			        
			        totalGeral += embarquesTotais;
	        	}
	        	
	        	row = sheet1.createRow(linha);
	        	HSSFCell cellTotal = row.createCell(0);
	        	cellTotal.setCellValue("TOTAL");
	        	HSSFCell cellTotalValue = row.createCell(7);
	        	cellTotalValue.setCellValue(totalGeral);
	        	
	        	linha = NUM_LINHA_INICIO;
	        	
	        	sheet1.autoSizeColumn(0);
	        	sheet1.autoSizeColumn(1);
	        	sheet1.autoSizeColumn(2);
	        	sheet1.autoSizeColumn(3);
	        	sheet1.autoSizeColumn(4);
	        	sheet1.autoSizeColumn(5);
	        	sheet1.autoSizeColumn(6);
	        	sheet1.autoSizeColumn(7);
	        	
	        }
	        String nomeArquivo =
	                UsuarioLogado.getUsuarioLogado().getNombusuario()
	                + "_" + Calendar.getInstance().getTime().getTime()
	                + "_" + "EmbarqueLocalidade";
	        
	        File fNomeArquivo = File.createTempFile(nomeArquivo, ".tmp"); 
	
	        FileOutputStream stream = new FileOutputStream(fNomeArquivo);
	        stream.flush();
	        wb.write(stream);
	        stream.close();
	
	        InputStream is = null;
	        is = new FileInputStream(fNomeArquivo);
	        Filedownload.save(is, "application/xls", "EmbarqueLocalidade.xls");
	
	        fNomeArquivo.delete();
		} catch (Exception e){
			log.error("", e);
		}
	}
	
	private void geraCabecalhoTitulo(HSSFSheet sheet1, HSSFWorkbook wb){
		HSSFRow titulo = sheet1.createRow(NUM_LINHA_TITULO);
		sheet1.addMergedRegion(new CellRangeAddress(
				NUM_LINHA_TITULO, //first row (0-based)
				NUM_LINHA_TITULO, //last row  (0-based)
	            0, //first column (0-based)
	            7  //last column  (0-based)
	    ));
		
		HSSFFont fonte = wb.createFont();
        fonte.setBoldweight(HSSFFont.BOLDWEIGHT_BOLD);
        HSSFCellStyle estilo = wb.createCellStyle();
        estilo.setFont(fonte);
        
        titulo.createCell(0).setCellValue("EMBARQUE POR LOCALIDADE");
		titulo.setRowStyle(estilo);
	}
	
	private void geraCabecalhoPeriodo(HSSFSheet sheet1){
		HSSFRow periodo = sheet1.createRow(NUM_LINHA_PERIODO);	
		SimpleDateFormat sdf = new SimpleDateFormat("dd/MM/yyyy");
		
		sheet1.addMergedRegion(new CellRangeAddress(
				NUM_LINHA_PERIODO, //first row (0-based)
				NUM_LINHA_PERIODO, //last row  (0-based)
	            0, //first column (0-based)
	            7  //last column  (0-based)
	    ));
		periodo.createCell(0).setCellValue("Perodo: " + sdf.format(this.datInicial.getValue()) + " a " + sdf.format(this.datFinal.getValue()) );
		
	}
	
	private void geraCabecalho(HSSFSheet sheet1, String dataAtual, HSSFWorkbook wb){
		
		geraCabecalhoTitulo(sheet1, wb);
		
		geraCabecalhoPeriodo(sheet1);	
		
		
		sheet1.addMergedRegion(new CellRangeAddress(
				NUM_LINHA_DATA_EMPRESA, //first row (0-based)
				NUM_LINHA_DATA_EMPRESA, //last row  (0-based)
	            1, //first column (0-based)
	            7  //last column  (0-based)
	    ));

		 //Montando o Cabealho
        HSSFRow r0 = sheet1.createRow(NUM_LINHA_DATA_EMPRESA);
        r0.createCell(0).setCellValue("DATA: " + dataAtual);
        r0.createCell(1).setCellValue("EMPRESA: " + empresa.getNombempresa());

        HSSFRow r1 = sheet1.createRow(NUM_LINHA_DATA_EMPRESA+1);
        r1.createCell(0).setCellValue("FAIXA HORRIA");
        r1.createCell(1).setCellValue("Prefixo / Nome Linha");
        r1.createCell(2).setCellValue("Embarques");
        r1.createCell(3).setCellValue("Prefixo / Nome Linha");
        r1.createCell(4).setCellValue("Embarques");
        r1.createCell(5).setCellValue("Prefixo / Nome Linha");
        r1.createCell(6).setCellValue("Embarques");
        r1.createCell(7).setCellValue("TOTAL");
	}

	public void onClick$btnExecutarRelatorio(Event ev) throws Exception {
		executarRelatorio();
	}

	@Override
	public void doAfterCompose(Component comp) throws Exception {
		lsEmpresas = empresaService.obtenerTodos();
		lsOrgaosConcedentes = orgaoConcedenteService.obtenerTodos();
		lsOrigens = new ArrayList<Parada>();
		lsRutas = new ArrayList<Ruta>();
		super.doAfterCompose(comp);
	}

	public void onSelect$cmbOrgaoConcedente(Event ev) throws Exception {
		lsRutas = rutaService.buscaSomenteRutasFromOrgao((OrgaoConcedente)cmbOrgaoConcedente.getSelectedItem().getValue());
		cmbRuta.setModel(new ListModelList(lsRutas));
	}
	
	public void onSelect$cmbRuta(Event ev) throws Exception {
		lsOrigens = paradaService.obtenerOrigenPorRuta((Ruta)cmbRuta.getSelectedItem().getValue());
		cmbOrigem.setModel(new ListModelList(lsOrigens));
	}
		
	protected String getSql(String empresa, String ruta, String orgao, String origem, String horInicio, String horFim) {
		StringBuilder sql = new StringBuilder();
		
		sql.append("select ");
		sql.append("	concat(concat(concat(TO_CHAR(b.fechorviaje,'DD/MM/YYYY HH24'), ':00'), ' a '),concat(TO_CHAR(b.fechorviaje,'HH24'), ':59')),  ");
		sql.append("	concat(concat(r.prefixo, ' '), r.descruta),  ");
		sql.append("	count(b.boleto_id)  ");
		sql.append("from  ");
		sql.append("	boleto b ");
		sql.append("	inner join ruta r on r.ruta_id = b.ruta_id ");
		sql.append("	inner join marca m on m.marca_id = b.marca_id ");
		sql.append("where  ");		
		sql.append("	b.fechorviaje between TO_DATE(:fecInicio, 'DD/MM/YYYY HH24:MI:SS') AND TO_DATE(:fecFinal, 'DD/MM/YYYY HH24:MI:SS')  ");
		if (!horInicio.isEmpty() && !horFim.isEmpty()){
			sql.append("    and TO_CHAR(b.fechorviaje, 'HH24:MI:SS') between :horInicio AND :horFinal  ");
		}
		sql.append("    and b.motivocancelacion_id is null and b.numasiento is not null and b.indstatusoperacion = 'F' ");
		if (orgao != null && !orgao.isEmpty()){
			sql.append("	and r.orgaoconcedente_id = :orgao ");
		}
		if (!empresa.isEmpty()){
			sql.append("	and m.empresa_id = :empresa ");
		}
		if (ruta != null && !ruta.isEmpty() && !ruta.equals("-1")){
			sql.append("	and r.ruta_id = :ruta ");
		}
		if (origem != null && !origem.isEmpty()){
			sql.append("    and b.origen_id = :origem ");
		}
		
		sql.append("group by ");
		sql.append("	concat(concat(concat(TO_CHAR(b.fechorviaje,'DD/MM/YYYY HH24'), ':00'), ' a '),concat(TO_CHAR(b.fechorviaje,'HH24'), ':59')),   ");
		sql.append("	concat(concat(r.prefixo, ' '), r.descruta) ");
		sql.append("order by ");
		sql.append("	1 "); 
		
		return sql.toString();
	}
	
	protected Map<String, Map<String, List<RelatorioEmbarqueLocalidadeBean>>> ResultSetToList(ResultSet rset) throws SQLException {
		
		Map<String, Map<String, List<RelatorioEmbarqueLocalidadeBean>>> datas = new LinkedHashMap<String, Map<String, List<RelatorioEmbarqueLocalidadeBean>>>();
		
		while (rset.next()) {			
			RelatorioEmbarqueLocalidadeBean bean = new RelatorioEmbarqueLocalidadeBean();
			bean.setData(rset.getString(1));
			bean.setLinha(rset.getString(2));
			bean.setEmbarques(rset.getLong(3));
			
			if (!datas.containsKey(bean.getData())){
				datas.put(bean.getData(), new LinkedHashMap<String, List<RelatorioEmbarqueLocalidadeBean>>());				
			}
			
			if (!datas.get(bean.getData()).containsKey(bean.getHorario())){
				datas.get(bean.getData()).put(bean.getHorario(), new ArrayList<RelatorioEmbarqueLocalidadeBean>());
			}
			
			datas.get(bean.getData()).get(bean.getHorario()).add(bean);
		}
		return datas;
	}
	
	public Datebox getDatInicial() {
		return datInicial;
	}

	public void setDatInicial(Datebox datInicial) {
		this.datInicial = datInicial;
	}

	public Datebox getDatFinal() {
		return datFinal;
	}

	public void setDatFinal(Datebox datFinal) {
		this.datFinal = datFinal;
	}

	public MyComboboxEstandar getCmbEmpresa() {
		return cmbEmpresa;
	}

	public void setCmbEmpresa(MyComboboxEstandar cmbEmpresa) {
		this.cmbEmpresa = cmbEmpresa;
	}

	public List<Empresa> getLsEmpresas() {
		return lsEmpresas;
	}

	public void setLsEmpresas(List<Empresa> lsEmpresas) {
		this.lsEmpresas = lsEmpresas;
	}

	public List<OrgaoConcedente> getLsOrgaosConcedentes() {
		return lsOrgaosConcedentes;
	}

	public void setLsOrgaosConcedentes(List<OrgaoConcedente> lsOrgaosConcedentes) {
		this.lsOrgaosConcedentes = lsOrgaosConcedentes;
	}

	public List<Ruta> getLsRutas() {
		return lsRutas;
	}

	public void setLsRutas(List<Ruta> lsRutas) {
		this.lsRutas = lsRutas;
	}

	public Timebox getHorInicial() {
		return horInicial;
	}

	public void setHorInicial(Timebox horInicial) {
		this.horInicial = horInicial;
	}

	public Timebox getHorFinal() {
		return horFinal;
	}

	public void setHorFinal(Timebox horFinal) {
		this.horFinal = horFinal;
	}

	public MyComboboxEstandar getCmbOrgaoConcedente() {
		return cmbOrgaoConcedente;
	}

	public void setCmbOrgaoConcedente(MyComboboxEstandar cmbOrgaoConcedente) {
		this.cmbOrgaoConcedente = cmbOrgaoConcedente;
	}

	public MyComboboxEstandar getCmbRuta() {
		return cmbRuta;
	}

	public void setCmbRuta(MyComboboxEstandar cmbRuta) {
		this.cmbRuta = cmbRuta;
	}

	public MyComboboxEstandar getCmbOrigem() {
		return cmbOrigem;
	}

	public void setCmbOrigem(MyComboboxEstandar cmbOrigem) {
		this.cmbOrigem = cmbOrigem;
	}

	public List<Parada> getLsOrigens() {
		return lsOrigens;
	}

	public void setLsOrigens(List<Parada> lsOrigens) {
		this.lsOrigens = lsOrigens;
	}

}
